/*
 * $Revision: 6107 $
 *
 * (C) Copyright 2001 Sun Microsystems, Inc.
 * Copyright (c) OSGi Alliance (2001, 2008). All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package java.lang;
public final class StringBuffer implements java.io.Serializable, java.lang.CharSequence {
	public StringBuffer() { }
	public StringBuffer(int var0) { }
	public StringBuffer(java.lang.String var0) { }
	public java.lang.StringBuffer append(char var0) { return null; }
	public java.lang.StringBuffer append(double var0) { return null; }
	public java.lang.StringBuffer append(float var0) { return null; }
	public java.lang.StringBuffer append(int var0) { return null; }
	public java.lang.StringBuffer append(long var0) { return null; }
	public java.lang.StringBuffer append(java.lang.Object var0) { return null; }
	public java.lang.StringBuffer append(java.lang.String var0) { return null; }
	public java.lang.StringBuffer append(java.lang.StringBuffer var0) { return null; }
	public java.lang.StringBuffer append(boolean var0) { return null; }
	public java.lang.StringBuffer append(char[] var0) { return null; }
	public java.lang.StringBuffer append(char[] var0, int var1, int var2) { return null; }
	public int capacity() { return 0; }
	public char charAt(int var0) { return 0; }
	public java.lang.StringBuffer delete(int var0, int var1) { return null; }
	public java.lang.StringBuffer deleteCharAt(int var0) { return null; }
	public void ensureCapacity(int var0) { }
	public void getChars(int var0, int var1, char[] var2, int var3) { }
	public int indexOf(java.lang.String var0) { return 0; }
	public int indexOf(java.lang.String var0, int var1) { return 0; }
	public java.lang.StringBuffer insert(int var0, char var1) { return null; }
	public java.lang.StringBuffer insert(int var0, double var1) { return null; }
	public java.lang.StringBuffer insert(int var0, float var1) { return null; }
	public java.lang.StringBuffer insert(int var0, int var1) { return null; }
	public java.lang.StringBuffer insert(int var0, long var1) { return null; }
	public java.lang.StringBuffer insert(int var0, java.lang.Object var1) { return null; }
	public java.lang.StringBuffer insert(int var0, java.lang.String var1) { return null; }
	public java.lang.StringBuffer insert(int var0, boolean var1) { return null; }
	public java.lang.StringBuffer insert(int var0, char[] var1) { return null; }
	public java.lang.StringBuffer insert(int var0, char[] var1, int var2, int var3) { return null; }
	public int lastIndexOf(java.lang.String var0) { return 0; }
	public int lastIndexOf(java.lang.String var0, int var1) { return 0; }
	public int length() { return 0; }
	public java.lang.StringBuffer replace(int var0, int var1, java.lang.String var2) { return null; }
	public java.lang.StringBuffer reverse() { return null; }
	public void setCharAt(int var0, char var1) { }
	public void setLength(int var0) { }
	public java.lang.CharSequence subSequence(int var0, int var1) { return null; }
	public java.lang.String substring(int var0) { return null; }
	public java.lang.String substring(int var0, int var1) { return null; }
}

